@@ -37,6 +37,8 @@ our %control_parameters : shared;
 "np"=>1,
 "ne"=>1,
 "nk"=>1,
+"ng"=>1,
+"ng_b"=>1,
 "ne_b"=>1,
 "natm"=>undef,
 "a_vector"=>undef,
@@ -148,7 +150,8 @@ sub main {
     }
     if($control_parameters{property} eq 'zeff' and $control_parameters{eval_zeff}) {
         print "-- evaluating Zeff --\n";
-        &exec_zeff;
+#        &exec_zeff;
+        &exec_zeff_via_dirlist;
         print "-- results --\n";
         &print_zeff(&get_newest("zeff","output[0-9][0-9][0-9]"));
     }
@@ -354,12 +357,14 @@ sub check_parameters {
     my $nd = $control_parameters{ndir};
     my $ne = $control_parameters{ne};
     my $nk = $control_parameters{nk};
+    my $ng = $control_parameters{ng};
+    my $ng_b = $control_parameters{ng_b};
     my $ne_b = $control_parameters{ne_b};
-    if( $np!=$nd*$ne*$nk ) {
-        &end("np is not equal to nd x ne x nk\n");
+    if( $np!=$nd*$ne*$nk*$ng ) {
+        &end("np is not equal to nd x ne x nk x ng\n");
     }
-    if( $np!=$nd*$ne_b ) {
-        &end("np is not equal to nd x ne_b\n");
+    if( $np!=$nd*$ne_b*$ng_b ) {
+        &end("np is not equal to nd x ne_b x ng_b\n");
     }
 
     if($control_parameters{property} eq "zeff" and !defined($control_parameters{atom_list})) {
@@ -606,10 +611,12 @@ sub read_control {
     print "number of directories to be handled in parallel : $control_parameters{ndir}\n";
     print "k-point parallelization for the SCF calculation      : $control_parameters{nk}\n";
     print "band-parallelization for the SCF calculation         : $control_parameters{ne}\n";
+    print "G-point parallelization for the SCF calculation      : $control_parameters{ng}\n";
     print "band-parallelization for the berry-phase calculation : $control_parameters{ne_b}\n";
+    print "G-point parallelization for the berry-phase calculation : $control_parameters{ng_b}\n";
     print "SCF   command : $control_parameters{scf_command}\n";
     print "berry command : $control_parameters{berry_command}\n";
-    print "(remark : NP will be replaced by np, NE will be replaced by ne, NK will be replaced by nk and NE_B will be replaced by ne_b)\n";
+    print "(remark : NP will be replaced by np, NE will be replaced by ne, NK will be replaced by nk, NG_B will be replaced by ng_b and NE_B will be replaced by ne_b)\n";
     print "\n";
 }
 
@@ -790,14 +797,16 @@ sub generate_scf_directory {
           &end("failed to open $sdir/$nfinp\n");
 
         my $line = get_input("$tmpldir/$nfinp");
-        $line = &strip_block($line,"displacement",1);
-        $line = &insert_block($line,"atom_list",$displacement_block,2);
         if($control_parameters{property} eq 'zeff'){
+            $line = &strip_block($line,"displacement",1);
+            $line = &insert_block($line,"atom_list",$displacement_block,2);
             $line =~s/\<ATOM_ID\>/$id/;
             $line =~s/\<Ux\>/0.0/;
             $line =~s/\<Uy\>/0.0/;
             $line =~s/\<Uz\>/0.0/;
         } elsif ($control_parameters{property} eq 'piezo') {
+            $line = &strip_block($line,"strain",1);
+            $line = &insert_block($line,"structure",$strain_block,1);
             $line =~s/\<E11\>/$e11/;
             $line =~s/\<E22\>/$e22/;
             $line =~s/\<E33\>/$e33/;
@@ -1081,8 +1090,10 @@ sub execute {
         my $comm = $command;
         $comm =~s/NP/$control_parameters{np}/;
         $comm =~s/NE_B/$control_parameters{ne_b}/;
+        $comm =~s/NG_B/$control_parameters{ng_b}/;
         $comm =~s/NE/$control_parameters{ne}/;
         $comm =~s/NK/$control_parameters{nk}/;
+        $comm =~s/NG/$control_parameters{ng}/;
         if ($ndir_per_cal==1) {
             print "running [$comm] under the following directories\n";
         } else {
@@ -1120,6 +1131,7 @@ sub execute {
             $comm =~s/NK//g;
             $comm =~s/ne//g;
             $comm =~s/nk//g;
+            $comm =~s/ng//g;
             $comm =~s/=//g;
             print "running [$comm] under the following directory\n";
             open(DIRLIST,">dirlist") or 
@@ -1351,6 +1363,43 @@ sub edit_file_names_data {
    return $acc;
 }
 
+sub exec_zeff_via_dirlist {
+    my $zeff = "zeff";
+    my $tmpldir = "scf_a0";
+    my $berrydata = "berry.data";
+    if(! -e $zeff) {
+        mkdir $zeff;
+    }
+    my $nfinp = &get_nfinp("$tmpldir/file_names.data");
+    copy "$tmpldir/file_names.data",$zeff or &end("failed to copy $tmpldir/file_names.data");
+    copy "$tmpldir/$nfinp",$zeff or &end("failed to copy $tmpldir/file_names.data");
+    copy "$berrydata",$zeff or &end("failed to copy $berrydata");
+    my $line = get_input("$tmpldir/$nfinp");
+    $line = &strip_block($line,"displacement",1);
+    $line = &strip_block($line,"postprocessing",0);
+    $line = &strip_block($line,"control",0);
+    $line = &insert_block($line,"",$zeff_ctrl_block,0);
+    $line = &insert_block($line,"",$zeff_pp_block,0);
+    $line = $line . "\n";
+    open(OUT,">$zeff/$nfinp");
+    print OUT $line;
+    close(OUT);
+
+    open(DIRLIST,">dirlist") or 
+      &end("failed to create the dirlist file\n"); 
+    print DIRLIST "1\n";
+    print DIRLIST "$zeff";
+    close(DIRLIST);
+
+    my $phase = $control_parameters{scf_command};
+    $phase =~s/NP/1/;
+    $phase =~s/NE/1/;
+    $phase =~s/NK/1/;
+    $phase =~s/NG/1/;
+    &run($phase);
+    unlink "dirlist";
+}
+
 sub exec_zeff {
     my $zeff = "zeff";
     my $tmpldir = "scf_a0";
@@ -1378,6 +1427,7 @@ sub exec_zeff {
     $phase =~s/NP/1/;
     $phase =~s/NE/1/;
     $phase =~s/NK/1/;
+    $phase =~s/NG/1/;
     &run($phase);
     chdir("..");
 }
