@@ -2,7 +2,7 @@ module m_ES_Mag_Constraint
 ! $Id: m_ES_Mag_Constraint.f90 574 2017-05-31 03:00:48Z jkoga $
 
   use m_Parallelization,      only : ista_kngp, iend_kngp, ierr, npes, mype, &
-       &                             mpi_comm_group
+       &                             MPI_CommGroup
   use m_PlaneWaveBasisSet,    only : ngabc, kgp
 
 
@@ -41,10 +41,11 @@ module m_ES_Mag_Constraint
 ! === KT_add === 2014/08/09
   use m_Crystal_Structure,   only : Global_Quantz_Axis_Fixed
 ! ============== 2014/08/09
+  use mpi
 
   implicit none
 
-  include 'mpif.h'
+!  include 'mpif.h'
   integer istatus(mpi_status_size)
 
   real(kind=DP) :: MagField_constrain_global(3)
@@ -81,7 +82,7 @@ contains
 
             MagField_constrain_global(:) = mag_constraint_lambda &
                  &                         *( MagMom(:) -mag_moment0_global(:) )
-            
+
             Do is=2, ndim_magmom
                vlhxc_l(1,1,is) = vlhxc_l(1,1,is) + MagField_constrain_global(is-1)
             End do
@@ -90,7 +91,7 @@ contains
       else
          if ( mype == 0 ) then
             MagMom(1) = ( chgq_l( 1,1,1 ) -chgq_l( 1,1,2 ) )*univol
-            
+
             MagField_constrain_global(1) = mag_constraint_lambda &
                  &                        *( MagMom(1) -mag_moment0_global(1) )
 
@@ -180,7 +181,7 @@ contains
             else
                MagMom( ixyz ) = RhoMag_on_atom( ia, 1 ) -RhoMag_on_atom( ia, 2 )
             endif
-            
+
             if ( mag_moment0_atoms_is_defined ) then
                cfactor( ixyz ) = mag_constraint_lambda /univol &
                     &               *( MagMom(ixyz) -mag_moment0_atoms(ia,ixyz) )
@@ -188,7 +189,7 @@ contains
                cfactor( ixyz ) = mag_constraint_lambda /univol &
                     &               *( MagMom(ixyz) -mag_moment0_atomtyp(it,ixyz) )
             endif
-               
+
             call calc_phase2( natm, pos, ia, kgp, ngabc, ista_kngp, iend_kngp, &
                  &            zfcos, zfsin )
 
@@ -202,10 +203,10 @@ contains
                VecG(1) = rltv(1,1)*ngabc(i,1) +rltv(1,2)*ngabc(i,2) +rltv(1,3)*ngabc(i,3)
                VecG(2) = rltv(2,1)*ngabc(i,1) +rltv(2,2)*ngabc(i,2) +rltv(2,3)*ngabc(i,3)
                VecG(3) = rltv(3,1)*ngabc(i,1) +rltv(3,2)*ngabc(i,2) +rltv(3,3)*ngabc(i,3)
-            
+
                normG = sqrt( VecG(1)**2 +VecG(2)**2 +VecG(3)**2 )
                normG3 = normG**3
-               
+
                fac1r = zfcos(i); fac1i = zfsin(i)
 !
                gr = normG *rad1
@@ -250,7 +251,7 @@ contains
 
     end subroutine case_constraint_moment_local
 
-    subroutine case_constraint_direc_local        
+    subroutine case_constraint_direc_local
       real(kind=DP) :: MagMom(3), cfactor(3), MagDirec(3)
       real(kind=DP) :: rad1, fac1r, fac1i, fac2
       real(kind=DP) :: VecG(3), normG, normG3, gr, d1
@@ -369,10 +370,10 @@ contains
                VecG(1) = rltv(1,1)*ngabc(i,1) +rltv(1,2)*ngabc(i,2) +rltv(1,3)*ngabc(i,3)
                VecG(2) = rltv(2,1)*ngabc(i,1) +rltv(2,2)*ngabc(i,2) +rltv(2,3)*ngabc(i,3)
                VecG(3) = rltv(3,1)*ngabc(i,1) +rltv(3,2)*ngabc(i,2) +rltv(3,3)*ngabc(i,3)
-            
+
                normG = sqrt( VecG(1)**2 +VecG(2)**2 +VecG(3)**2 )
                normG3 = normG**3
-               
+
                fac1r = zfcos(i); fac1i = zfsin(i)
 !
                gr = normG *rad1
@@ -456,7 +457,7 @@ contains
 
          ig = iproj_group(ia)
          if(ig<1) cycle
-            
+
          Do ixyz=1, ixyz_max
             if ( noncol ) then
                c1 = 0.0d0
@@ -492,15 +493,15 @@ contains
                ip = proj_group(i,ig)
                it = proj_attribute(ip)%ityp
                ilp = proj_attribute(ip)%l+1
-               
+
                do ilmt1 = 1, ilmt(it)
                   l1 = ltp(ilmt1,it); m1 = mtp(ilmt1,it); t1 = taup(ilmt1,it)
                   if ( l1 /= ilp ) cycle
-                  
+
                   do ilmt2 = 1, ilmt(it)
                      l2 = ltp(ilmt2,it); m2 = mtp(ilmt2,it); t2 = taup(ilmt2,it)
                      if( l2 /= ilp ) cycle
-                     
+
                      if ( m1 == m2 ) then
                         MagField_constrain_hardpart(ilmt1,ilmt2,ia,ixyz) &
                              & = MagField_constrain_hardpart(ilmt1,ilmt2,ia,ixyz) &
@@ -564,10 +565,17 @@ contains
                Do ixyz=1, 3
                   c1 = cnorm2 - MagMom(ixyz)**2
                   c2 = cnorm**3
-                  MagField_constrain_hardpart(lmt1,lmt2,ia,ixyz) = &
-                       &  mag_constraint_lambda *c1 /c2 &
-                       & *( MagDirec(ixyz) -mag_direction0_atomtyp(it,ixyz) ) &
-                       & *dl2p(lmt1,lmt2,1,it)             ! approx
+                  if ( mag_moment0_atoms_is_defined ) then
+                     MagField_constrain_hardpart(lmt1,lmt2,ia,ixyz) = &
+                          &  mag_constraint_lambda *c1 /c2 &
+                          & *( MagDirec(ixyz) -mag_direction0_atoms(ia,ixyz) ) &
+                          & *dl2p(lmt1,lmt2,1,it)             ! approx
+                  else
+                     MagField_constrain_hardpart(lmt1,lmt2,ia,ixyz) = &
+                          &  mag_constraint_lambda *c1 /c2 &
+                          & *( MagDirec(ixyz) -mag_direction0_atomtyp(it,ixyz) ) &
+                          & *dl2p(lmt1,lmt2,1,it)             ! approx
+                  endif
                End do
             End do
          End do
@@ -625,7 +633,7 @@ contains
                MagMom(is-1) = chgq_l(  1,1,is ) *univol
 !               MagMomOld(is-1) = chgqo_l(  1,1,is ) *univol
             End do
-            
+
             c1 = 0.0d0;  c2 = 0.0d0
             Do ixyz=1, 3
                c1 = c1 + MagField_constrain_global(ixyz) *MagMom(ixyz)
@@ -633,7 +641,7 @@ contains
             End do
          else
             MagMom(1) = ( chgq_l( 1,1,1 ) -chgq_l( 1,1,2 ) )*univol
-            
+
             c1 = MagField_constrain_global(1) *MagMom(1)
             c2 = ( MagMom(1) -mag_moment0_global(1) )**2
          endif
@@ -643,10 +651,10 @@ contains
       endif
 
       if (npes > 1) then
-         call mpi_bcast( ene_double_counting, 1, mpi_double_precision, & 
-              &          0, mpi_comm_group, ierr )
+         call mpi_bcast( ene_double_counting, 1, mpi_double_precision, &
+              &          0, MPI_CommGroup, ierr )
          call mpi_bcast( ene_mag_constraint,  1, mpi_double_precision, &
-              &          0, mpi_comm_group, ierr )
+              &          0, MPI_CommGroup, ierr )
       end if
 
     end subroutine case_constraint_moment_global
@@ -684,10 +692,10 @@ contains
       endif
 
       if (npes > 1) then
-         call mpi_bcast( ene_double_counting, 1, mpi_double_precision, & 
-              &          0, mpi_comm_group, ierr )
+         call mpi_bcast( ene_double_counting, 1, mpi_double_precision, &
+              &          0, MPI_CommGroup, ierr )
          call mpi_bcast( ene_mag_constraint,  1, mpi_double_precision, &
-              &          0, mpi_comm_group, ierr )
+              &          0, MPI_CommGroup, ierr )
       end if
 
     end subroutine case_constraint_direc_global
@@ -738,9 +746,15 @@ contains
 
             if ( cnorm4 > 0.0 .and. cnorm > cnorm_lower_limit ) then
                MagDirec = MagMom / cnorm
-               Do ixyz=1, 3
-                  csum2 = csum2 + ( MagDirec(ixyz) -mag_direction0_atomtyp(it,ixyz) )**2
-               End Do
+               if ( mag_moment0_atoms_is_defined ) then
+                  Do ixyz=1, 3
+                     csum2 = csum2 +( MagDirec(ixyz)-mag_direction0_atoms(ia,ixyz) )**2
+                  End Do
+               else
+                  Do ixyz=1, 3
+                     csum2 = csum2 +( MagDirec(ixyz)-mag_direction0_atomtyp(it,ixyz) )**2
+                  End Do
+               endif
             endif
          End do
 
@@ -793,7 +807,7 @@ contains
 
       if ( npes > 1 ) then
          call mpi_allreduce( csum1, c1, 1, mpi_double_precision, &
-              &              mpi_sum, mpi_comm_group, ierr )
+              &              mpi_sum, MPI_CommGroup, ierr )
          csum1 = c1
       endif
 
@@ -859,7 +873,7 @@ contains
 
       if ( npes > 1 ) then
          call mpi_allreduce( csum1, c1, 1, mpi_double_precision, &
-              &              mpi_sum, mpi_comm_group, ierr )
+              &              mpi_sum, MPI_CommGroup, ierr )
          csum1 = c1
       endif
 
@@ -888,7 +902,7 @@ contains
                Do ilmt1=1, ilmt(it)
                   Do ilmt2=1, ilmt(it)
                      csum1 = csum1 +MagField_constrain_hardpart(ilmt1,ilmt2,ia,ixyz) &
-                          &          *hsr(ia,ilmt1,ilmt2,ixyz+1) 
+                          &          *hsr(ia,ilmt1,ilmt2,ixyz+1)
                   End do
                End do
             End do
@@ -899,7 +913,7 @@ contains
             Do ilmt1=1, ilmt(it)
                Do ilmt2=1, ilmt(it)
                   csum1 = csum1 +MagField_constrain_hardpart(ilmt1,ilmt2,ia,1) &
-                       &          *( hsr(ia,ilmt1,ilmt2,1) -hsr(ia,ilmt1,ilmt2,2) ) 
+                       &          *( hsr(ia,ilmt1,ilmt2,1) -hsr(ia,ilmt1,ilmt2,2) )
                End Do
             End do
          End do
@@ -907,10 +921,10 @@ contains
 
       Do ia=1, natm
          it = ityp(ia)
-            
+
          ig = iproj_group(ia)
          if(ig<1) cycle
-            
+
          if ( noncol ) then
             Do ixyz=1, ixyz_max
                c1 = 0.0d0
@@ -945,7 +959,7 @@ contains
       End Do
       ene_double_counting = csum1
       ene_mag_constraint = csum2 *mag_constraint_lambda/ 2.0d0
-      
+
     end subroutine case_constraint_moment_occmat
 
     subroutine case_constraint_direc_hardpart
@@ -983,9 +997,15 @@ contains
                End do
             End do
 
-            Do ixyz=1, 3
-               csum2 = csum2 + ( MagDirec(ixyz)-mag_direction0_atomtyp(it,ixyz) )**2
-            End do
+            if ( mag_moment0_atoms_is_defined ) then
+               Do ixyz=1, 3
+                  csum2 = csum2 + ( MagDirec(ixyz)-mag_direction0_atoms(ia,ixyz) )**2
+               End do
+            else
+               Do ixyz=1, 3
+                  csum2 = csum2 + ( MagDirec(ixyz)-mag_direction0_atomtyp(it,ixyz) )**2
+               End do
+            endif
          endif
 
       End do
@@ -1042,7 +1062,7 @@ contains
 
     Do is=2, ndim_magmom
        chgq_l(:,:,is) = 0.0d0
-       hsr(:,:,:,is) = 0.0d0;    
+       hsr(:,:,:,is) = 0.0d0;
 !!!!       hsi(:,:,:,is) = 0.0d0
     End do
   end subroutine m_ES_set_magmom_zero
