@@ -1,6 +1,6 @@
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2014.03 ($Rev: 409 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 570 $)
 !
 !  PROGRAM: TDLRMAIN
 !
@@ -12,13 +12,13 @@
 !=======================================================================
 
 ! ======================================================================
-!  This is a module for setting arrays used in the BS-like equation 
+!  This is a module for setting arrays used in the BS-like equation
 !  for the LR-TDDFT .
 ! ======================================================================
 
 ! ======================= history ======================================
 !  ver 1.0 :  2010/3/31
-!               applicable to the crystal such as Si.  
+!               applicable to the crystal such as Si.
 !  ver 2.0 :  2011/3/31
 !               applicable to the isolated molecule such as C6H6.
 !
@@ -35,7 +35,7 @@ module m_LinearResponse_BS
        &                  ista_fftp, iend_fftp, &
        &                  ista_fftph, iend_fftph, &
        &                  np_e, ista_k, iend_k, map_e, map_z, myrank_e, &
-       &                  mpi_comm_group
+       &                  MPI_CommGroup
 
   use m_Control_Parameters,  only : Num_q_Points, way_of_smearing, neg, &
        &                            ipri, nspin, printable, xctype, kimg
@@ -67,7 +67,7 @@ module m_LinearResponse_BS
 
   use m_LinearResponse_Control,     only  : nrd_efermi, nstep, e, nmax_G_LR, &
        &                                    vqxyz,  &
-       &                                    xc_kernel_type, RPA, ALDA_R, & 
+       &                                    xc_kernel_type, RPA, ALDA_R, &
        &                                    scissor, eta, &
        &                                    sw_LongWaveLimit
 
@@ -189,7 +189,7 @@ contains
     End do
 ! ----------------------------- end ------------
     if ( mype == 0 ) then
-       MatK = MatK * univol / dble(ngrid) 
+       MatK = MatK * univol / dble(ngrid)
 !!!!!!!!!!!!!!!!!!!!!!!       MatK = MatK*2.0d0        ! ???? bug ??
     endif
 ! --
@@ -209,7 +209,7 @@ contains
       allocate( chg2r(ista_fftph:iend_fftph) ); chg2r = 0.0d0
       allocate( chg2i(ista_fftph:iend_fftph) ); chg2i = 0.0d0
 !
-      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, band_start_LR: band_end_LR ) ); 
+      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, band_start_LR: band_end_LR ) );
       Wfns_on_Rspace = 0.0d0
       Do ib1 = band_start_LR, band_end_LR
          Call  Get_WF_in_Rspace_CD( ik, ib1, wfn_k, wf1_tmp, &
@@ -221,7 +221,7 @@ contains
 !      c1 = 0.0d0
 !      DO jj=ista_fftph,iend_fftph
 !         c1 = c1 + Wfns_on_Rspace( 2*jj-1, band_start_LR )**2 &
-!              &  + Wfns_on_Rspace( 2*jj,   band_start_LR )**2 
+!              &  + Wfns_on_Rspace( 2*jj,   band_start_LR )**2
 !      End do
 !      write(*,*) 'c1 = ', c1, c1 / dble(iend_fftph-ista_fftph+1), c1 * univol, c1 * univol /dble(iend_fftph-ista_fftph+1)           ! c1/(iend-isa+1) == 1 deshita.
 ! ---------------
@@ -235,10 +235,10 @@ contains
       deallocate( chg1r, chg1i ); deallocate( chg2r, chg2i )
       deallocate( Wfns_On_Rspace )
     end subroutine unset_work_arrays
-    
+
     subroutine set_chg1
       integer ::i, ip
-      
+
       wf1_tmp( ista_fftp:iend_fftp ) = Wfns_on_Rspace( ista_fftp:iend_fftp, ib1 )
       wf2_tmp( ista_fftp:iend_fftp ) = Wfns_on_Rspace( ista_fftp:iend_fftp, ib2 )
       Do i=ista_fftph, iend_fftph
@@ -247,10 +247,10 @@ contains
          chg1i(i) = wf1_tmp(ip) *wf2_tmp(ip+1) - wf1_tmp(ip+1) *wf2_tmp(ip)
       End do
     end subroutine set_chg1
-     
+
     subroutine set_chg2
       integer ::i, ip
-      
+
       wf1_tmp( ista_fftp:iend_fftp ) = Wfns_on_Rspace( ista_fftp:iend_fftp, ib3 )
       wf2_tmp( ista_fftp:iend_fftp ) = Wfns_on_Rspace( ista_fftp:iend_fftp, ib4 )
       Do i=ista_fftph, iend_fftph
@@ -274,7 +274,7 @@ contains
       End do
       if ( npes > 1 ) then
          call mpi_allreduce( ztmp, zsum, 2, MPI_DOUBLE_PRECISION, &
-              &              MPI_SUM, mpi_comm_group, ierr )
+              &              MPI_SUM, MPI_CommGroup, ierr )
          ztmp = zsum
       endif
       if ( mype==0 ) MatK(ix1,ix2) = ztmp
@@ -310,13 +310,13 @@ contains
        endif
 
        call set_rho1_tmp
-       
+
        Do ix2=1, matsize
           if ( sw_NODA == ON .and. ix1 /= ix2 ) cycle
 
           ib3 = ( ix2-1 ) /nband_LR +1
           ib4 = mod( ix2-1,nband_LR ) + 1
-          
+
           ib3 = ib3 + band_start_LR - 1;     ib4 = ib4 + band_start_LR - 1
 
           call set_rho2_tmp
@@ -348,7 +348,7 @@ contains
       endif
       if ( npes > 1 ) then
          call mpi_bcast( rho1_tmp, 2*kg1*kv3_ek, MPI_DOUBLE_PRECISION, &
-              &         map_e(ib2), mpi_comm_group, ierr )
+              &         map_e(ib2), MPI_CommGroup, ierr )
       endif
     end subroutine set_rho1_tmp
 
@@ -361,7 +361,7 @@ contains
       endif
       if ( npes > 1 ) then
          call mpi_bcast( rho2_tmp, 2*kg1*kv3_ek, MPI_DOUBLE_PRECISION, &
-              &         map_e(ib4), mpi_comm_group, ierr )
+              &         map_e(ib4), MPI_CommGroup, ierr )
       endif
     end subroutine set_rho2_tmp
 
@@ -395,7 +395,7 @@ contains
 
 !--------------------------------------------------------
 !!
-!!!        Matrix L0    for   BS && ALDA_R,   LongWaveLimit 
+!!!        Matrix L0    for   BS && ALDA_R,   LongWaveLimit
 !!
 !---------------------------------------------------------
   subroutine Calc_MatL0( e_value )
@@ -508,7 +508,7 @@ contains
 
     subroutine set_work_arrays
       allocate( wf1_tmp(ista_fftp:iend_fftp) ); wf1_tmp = 0.0d0
-      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, 1:neg ) ); 
+      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, 1:neg ) );
       Wfns_on_Rspace = 0.0d0
       Do ib1 = 1, neg
          Call  Get_WF_in_Rspace_CD( ik, ib1, wfn_k, wf1_tmp, &
@@ -522,7 +522,7 @@ contains
       deallocate( wf1_tmp )
       deallocate( Wfns_On_Rspace )
     end subroutine unset_work_arrays
-    
+
   End subroutine Check_Charge_on_mesh
 
   subroutine Check_Exc_energy
@@ -566,7 +566,7 @@ contains
 
     subroutine set_work_arrays
       allocate( wf1_tmp(ista_fftp:iend_fftp) ); wf1_tmp = 0.0d0
-      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, 1:neg ) ); 
+      allocate( Wfns_on_Rspace( ista_fftp:iend_fftp, 1:neg ) );
       Wfns_on_Rspace = 0.0d0
       Do ib1 = 1, neg
          Call  Get_WF_in_Rspace_CD( ik, ib1, wfn_k, wf1_tmp, &
@@ -580,7 +580,7 @@ contains
       deallocate( wf1_tmp )
       deallocate( Wfns_On_Rspace )
     end subroutine unset_work_arrays
-    
+
   End subroutine Check_Exc_energy
 
 ! --
@@ -603,7 +603,7 @@ contains
           endif
 !!!       endif
     End do
-   if ( npes > 1 ) call mpi_barrier( mpi_comm_group,ierr )
+   if ( npes > 1 ) call mpi_barrier( MPI_CommGroup,ierr )
   end subroutine Print_MatK
 
   subroutine Print_MatV
@@ -625,7 +625,7 @@ contains
           endif
 !!!!       endif
     End do
-    if ( npes > 1 ) call mpi_barrier( mpi_comm_group,ierr )
+    if ( npes > 1 ) call mpi_barrier( MPI_CommGroup,ierr )
   end subroutine Print_MatV
 ! -----------------
 end module m_LinearResponse_BS
